@@ -1,7 +1,8 @@
 /*
  *  Off-the-Record Messaging library
- *  Copyright (C) 2004-2012  Ian Goldberg, Rob Smits, Chris Alexander,
- *  			      Willy Lew, Lisa Du, Nikita Borisov
+ *  Copyright (C) 2004-2014  Ian Goldberg, David Goulet, Rob Smits,
+ *                           Chris Alexander, Willy Lew, Lisa Du,
+ *                           Nikita Borisov
  *                           <otr@cypherpunks.ca>
  *
  *  This library is free software; you can redistribute it and/or
@@ -30,6 +31,7 @@
 #include "serial.h"
 #include "proto.h"
 #include "context.h"
+#include "mem.h"
 
 #if OTRL_DEBUGGING
 #include <stdio.h>
@@ -370,7 +372,11 @@ gcry_error_t otrl_auth_handle_commit(OtrlAuthInfo *auth,
 	     * while in case some other logged in instance of our buddy
 	     * replied with a DHKEY message.  In that case, use the
 	     * incoming parameters.  Otherwise, compare the hashgx
-	     * values to see which one wins. */
+	     * values to see which one wins.
+	     *
+	     * This does NOT use constant time comparison because these
+	     * are two public values thus don't need it. Also, this checks
+	     * which pubkey is larger and not if they are the same. */
 	    if (!is_master && memcmp(auth->hashgx, hashbuf, 32) > 0) {
 		/* Ours wins.  Ignore the message we received, and just
 		 * resend the same D-H Commit message again. */
@@ -976,7 +982,9 @@ gcry_error_t otrl_auth_handle_revealsig(OtrlAuthInfo *auth,
 	    /* Check the hash */
 	    gcry_md_hash_buffer(GCRY_MD_SHA256, hashbuf, gxbuf,
 		    auth->encgx_len);
-	    if (memcmp(hashbuf, auth->hashgx, 32)) goto decfail;
+	    /* This isn't comparing secret data, but may as well use the
+	     * constant-time version. */
+	    if (otrl_mem_differ(hashbuf, auth->hashgx, 32)) goto decfail;
 
 	    /* Extract g^x */
 	    bufp = gxbuf;
@@ -1005,7 +1013,7 @@ gcry_error_t otrl_auth_handle_revealsig(OtrlAuthInfo *auth,
 	    gcry_md_reset(auth->mac_m2);
 	    gcry_md_write(auth->mac_m2, authstart, authend - authstart);
 
-	    if (memcmp(macstart,
+	    if (otrl_mem_differ(macstart,
 			gcry_md_read(auth->mac_m2, GCRY_MD_SHA256),
 			20)) goto invval;
 
@@ -1121,7 +1129,7 @@ gcry_error_t otrl_auth_handle_signature(OtrlAuthInfo *auth,
 	    /* Check the MAC */
 	    gcry_md_reset(auth->mac_m2p);
 	    gcry_md_write(auth->mac_m2p, authstart, authend - authstart);
-	    if (memcmp(macstart,
+	    if (otrl_mem_differ(macstart,
 			gcry_md_read(auth->mac_m2p, GCRY_MD_SHA256),
 			20)) goto invval;
 
