@@ -1,7 +1,8 @@
 /*
  *  Off-the-Record Messaging library
- *  Copyright (C) 2004-2012  Ian Goldberg, Rob Smits, Chris Alexander,
- *  			      Willy Lew, Lisa Du, Nikita Borisov
+ *  Copyright (C) 2004-2014  Ian Goldberg, David Goulet, Rob Smits,
+ *                           Chris Alexander, Willy Lew, Lisa Du,
+ *                           Nikita Borisov
  *                           <otr@cypherpunks.ca>
  *
  *  This library is free software; you can redistribute it and/or
@@ -242,10 +243,10 @@ char *otrl_proto_default_query_msg(const char *ourname, OtrlPolicy policy)
      * get passed to the main IM application for processing (and
      * free()ing). */
     const char *format = "?OTR%s\n<b>%s</b> has requested an "
-	    "<a href=\"http://otr.cypherpunks.ca/\">Off-the-Record "
+	    "<a href=\"https://otr.cypherpunks.ca/\">Off-the-Record "
 	    "private conversation</a>.  However, you do not have a plugin "
-	    "to support that.\nSee <a href=\"http://otr.cypherpunks.ca/\">"
-	    "http://otr.cypherpunks.ca/</a> for more information.";
+	    "to support that.\nSee <a href=\"https://otr.cypherpunks.ca/\">"
+	    "https://otr.cypherpunks.ca/</a> for more information.";
 
     /* Figure out the version tag */
     v1_supported = (policy & OTRL_POLICY_ALLOW_V1);
@@ -294,13 +295,16 @@ unsigned int otrl_proto_query_bestversion(const char *otrquerymsg,
 
 
     otrtag = strstr(otrquerymsg, "?OTR");
+    if (!otrtag) {
+	return 0;
+    }
     otrtag += 4;
 
-    if (otrtag && *otrtag == '?') {
+    if (*otrtag == '?') {
 	query_versions = (1<<0);
 	++otrtag;
     }
-    if (otrtag && *otrtag == 'v') {
+    if (*otrtag == 'v') {
 	for(++otrtag; *otrtag && *otrtag != '?'; ++otrtag) {
 	    switch(*otrtag) {
 		case '2':
@@ -463,6 +467,7 @@ gcry_error_t otrl_proto_instance(const char *otrmsg,
     free(bufp_head);
     return gcry_error(GPG_ERR_NO_ERROR);
 invval:
+    free(bufp_head);
     err = gcry_error(GPG_ERR_INV_VALUE);
     return err;
 }
@@ -823,7 +828,8 @@ gcry_error_t otrl_proto_accept_data(char **plaintextp, OtrlTLV **tlvsp,
 
     gcry_md_reset(sess->rcvmac);
     gcry_md_write(sess->rcvmac, macstart, macend-macstart);
-    if (memcmp(givenmac, gcry_md_read(sess->rcvmac, GCRY_MD_SHA1), 20)) {
+    if (otrl_mem_differ(givenmac, gcry_md_read(sess->rcvmac, GCRY_MD_SHA1),
+	    20)) {
 	/* The MACs didn't match! */
 	goto conflict;
     }
@@ -990,13 +996,19 @@ gcry_error_t otrl_proto_fragment_create(int mms, int fragment_count,
 {
     char *fragdata;
     int fragdatalen = 0;
-    unsigned short curfrag = 0;
+    int curfrag = 0;
     int index = 0;
     int msglen = strlen(message);
     /* Should vary by number of msgs */
     int headerlen = context->protocol_version == 3 ? 37 : 19;
 
-    char **fragmentarray = malloc(fragment_count * sizeof(char*));
+    char **fragmentarray;
+
+    if (fragment_count < 1 || fragment_count > 65535) {
+	return gcry_error(GPG_ERR_INV_VALUE);
+    }
+
+    fragmentarray = malloc(fragment_count * sizeof(char*));
     if(!fragmentarray) return gcry_error(GPG_ERR_ENOMEM);
 
     /*
@@ -1034,13 +1046,15 @@ gcry_error_t otrl_proto_fragment_create(int mms, int fragment_count,
 	 */
 	if (context->auth.protocol_version != 3) {
 	    snprintf(fragmentmsg, fragdatalen + headerlen,
-		    "?OTR,%05hu,%05hu,%s,", curfrag, fragment_count, fragdata);
+		    "?OTR,%05hu,%05hu,%s,", (unsigned short)curfrag,
+			    (unsigned short)fragment_count, fragdata);
 	} else {
 	    /* V3 messages require instance tags in the header */
 	    snprintf(fragmentmsg, fragdatalen + headerlen,
 		    "?OTR|%08x|%08x,%05hu,%05hu,%s,",
-		    context->our_instance, context->their_instance, curfrag,
-		    fragment_count, fragdata);
+		    context->our_instance, context->their_instance,
+		    (unsigned short)curfrag, (unsigned short)fragment_count,
+		    fragdata);
 	}
 	fragmentmsg[fragdatalen + headerlen] = 0;
 
